#!/usr/bin/env python
# -*- coding: UTF-8 -*-
u"""
utilitaire d'instanciation ligne de commande de fichiers templates à partir d'un dictionnaire xml

usage::

    %prog -x <path>/eole.xml -o <path>/test.txt -i <path>/dico.eol testtemplate.tmpl

testtemplate est le fichier à instancier

"""
import sys
import os.path as osp
from optparse import OptionParser
from creole.creolecat import instance
from creole.config import configeol

def parse_cmdline():
    parser = OptionParser(usage=__doc__)
    parser.add_option("-x", "--xml", dest="xmlfile", default=None,
                  help=u"dictionnaire xml (facultatif)", metavar="XMLFILE")

    parser.add_option("-o", "--output", dest="outputfile",
                  help=u"fichier généré", metavar="OUTPUTFILE")

    parser.add_option("-i", "--input", dest="inifile", default=configeol,
                  help=u"fichier de valeurs (.eol)", metavar="INPUTFILE")

    (options, args) = parser.parse_args()

    if not options.outputfile:
        parser.error("il faut un nom de fichier de sortie")

    if not len(args) == 1:
        parser.error("il faut un et un seul fichier de template")

    if args[0] == options.outputfile:
        parser.error("les fichiers source et destination doivent être différent")

    return options, args

def parse():
    options, args = parse_cmdline()
    inputfile = args[0]
    if not osp.isfile(inputfile):
        sys.exit("Template non trouvé : %s" % inputfile)
    xmlfile = options.xmlfile
    if xmlfile != None:
        for dic_xml in xmlfile.split(','):
            if not osp.exists(dic_xml):
                sys.exit("Fichier ou répertoire non existant : %s" % dic_xml)
    outputfile = options.outputfile
    inifile = options.inifile
    if inifile and not osp.isfile(inifile):
        sys.exit("Fichier de configuration non existant : %s" % inifile)
    return inputfile, xmlfile, outputfile, inifile

def main():
    try:
        instance(*parse())
    except Exception, e:
        sys.exit(str(e))

if __name__ == '__main__':
    main()
