#!/usr/bin/env python
# -*- coding: UTF-8 -*-
import optparse
import os, sys, tempfile
import gtk, gobject
import gtk.glade

from os.path import isfile
from creole.error import VirtError
try:
    from creole import config
    if not config.VIRTDISABLED and not isfile(config.containers_file):
        raise VirtError('Veuillez lancer gen_conteneurs avant gen_config')
except VirtError, e:
    print e
    sys.exit(1)

from creole import eosfunc
from creole.cfgparser import EoleDict
from creole.error import TypeEoleError
from xmlrpclib import base64, ProtocolError
REGISTRED = 0
try:
    from zephir.zephir_conf.zephir_conf import adresse_zephir, id_serveur
    from zephir.lib_zephir import EoleProxy as ServerProxy
    REGISTRED = 1
except:
    from xmlrpclib import ServerProxy
    adresse_zephir = None
    id_serveur = -1

GLADE_FILE = os.path.join(config.datadir, 'gen_config.glade')
EXPERT_COLOR = '#008000'

class Widgets:
    def __init__(self, text_box, label_box, button_box, sep_box=None):
        self.text_box = text_box
        self.label_box = label_box
        self.button_box = button_box
        self.sep_box = sep_box

    def destroy(self):
        if self.text_box: self.text_box.destroy()
        if self.label_box: self.label_box.destroy()
        if self.button_box: self.button_box.destroy()
        if self.sep_box: self.sep_box.destroy()

class ConfigGui:
    """
    Classe interface graphique pour configuration
    """

    def __init__(self, interface_file, dir_config=config.eoledirs,
                 mode='normal', window_mode=None, filename=None, title=None):
        self.dir_config = dir_config
        # parsage du fichier de theme
        gtk.rc_parse(os.path.join(config.datadir, 'eole.rc'))
        self.init_default_dict()
        self.interface_file = interface_file
        self.glade = gtk.glade.XML(interface_file,'main')
        self.glade_select = None
        self.window = self.glade.get_widget('main')
        # mode par défaut
        self.mode = mode
        # proxy xml-rpc vers Zephir
        self.zephir = None
        self.addr_zephir = None
        self.login_zephir = None
        # n° du serveur zephir en cours de traitement
        self.zephir_server = None
        self.zephir_modules = {}
        self.zephir_rne = None
        self.dial_select = None
        # variable cachées / grisées
        self.hide_vars = True
        # module du serveur local
        self.module_import = None
        #table contenant les edits + labels a droite
        self.table = self.glade.get_widget('table')
        #image de titre
        self.image = self.glade.get_widget('titre')
        #vbox de boutons des noms de familles
        self.vbox_button = self.glade.get_widget('vbox_button')

        sig = {'on_main_delete_event' : self._callback_button_quit,
               'on_quitter_activate' : self._callback_button_quit,
               'on_button_valid_group_clicked' : self._save_values,
               'on_button_load_default_clicked' : self._callback_load_defaults,
               'on_charger_defauts_activate' : self._callback_button_defaults,
               'on_enregistrer_activate' : self._callback_button_save,
               'on_ouvrir_activate' : self._callback_button_load,
               'on_import_creole1_activate' : self._callback_button_import,
               'on_normal_activate' : self._change_mode,
               'on_hide_vars_activate' : self._change_affvar,
               'on_zephir_get_activate' : self.zephir_get,
               'on_zephir_send_activate' : self.zephir_send,
               'on_zephir_connect_activate' : self.zephir_connect,
               'on_zephir_disconnect_activate' : self.zephir_disconnect,
               'on_zephir_select_activate' : self.zephir_select,
               'on_importcreole1_activate' : self.zephir_import
              }
        if window_mode:
            self.window.resize(800, 600)
        else:  # For use with framebuffer
            self.window.fullscreen()
            self.window.maximize()
            screen = self.window.get_screen()
            self.window.set_size_request(screen.get_width(),
                                         screen.get_height())
        self.glade.get_widget('zephir_send').set_sensitive(False)
        self.glade.get_widget('zephir_get').set_sensitive(False)
        self.glade.get_widget('zephir_select').set_sensitive(False)
        self.glade.get_widget('zephir_disconnect').set_sensitive(False)
        self.glade.get_widget('zephir_import').set_sensitive(False)
        self.glade.signal_autoconnect(sig)
        # restauration des imports Creole (#2103)
        #self.glade.get_widget('import_creole1').hide()
        #self.glade.get_widget('zephir_import').hide()
        # dictionnaire par defaut
        if title:
            self.set_title(title)
        else:
            self.set_title(config.eole_module.capitalize())
        if os.path.isfile(config.configeol):
            self.filename = config.configeol
        else:
            self.filename = '/root/zephir.eol'
        if filename != None and os.path.isfile(filename):
            if self.eole_dico.load_values(filename, check=False) != False:
                self.filename = filename
                self.window.set_title('Configuration: %s' % self.filename)
                self.valid_families = []
                self.main()
                self._refresh_display()
        # si enregistré sur zephir : passage en mode distant si demandé
        elif REGISTRED:
            self.zephir_connect()

    def init_default_dict(self):
        #pour stocker les edits et les labels
        self.creolevars = {}
        #pour stocker les boutons de familles
        self.button_stock = {}
        # dictionnaire des variables en erreur
        self.var_status = {}
        #flag pour savoir si on a sauve avant de quitter
        self.save_flag = True
        self.valid_families = []
        #variable indiquant la famille en cours d'édition
        self.edited_family = ""
        #si une case a été éditée (éviter la confirmation à chaque chgmt de famille
        self.modified_flag = False
        self.eole_dico = EoleDict(templatedir=config.templatedir)
        # chargement des dictionnaires
        self.eole_dico.read_dir(self.dir_config)

    def zephir_connect(self, *args):
        # saisie login/mdp
        glade_zeph = gtk.glade.XML(self.interface_file,'dialog_zeph')
        dial_zeph = glade_zeph.get_widget('dialog_zeph')
        focus = 'pwd_zeph'
        if self.login_zephir is not None:
            glade_zeph.get_widget('login_zeph').set_text(self.login_zephir)
        else:
            focus = 'login_zeph'
        if self.addr_zephir is not None:
            glade_zeph.get_widget('adresse_zeph').set_text(self.addr_zephir)
        elif adresse_zephir is not None:
            glade_zeph.get_widget('adresse_zeph').set_text(adresse_zephir)
        else:
            focus = 'adresse_zeph'
        glade_zeph.get_widget(focus).grab_focus()
        response = dial_zeph.run()
        if response == gtk.RESPONSE_OK:
            # récupération login/mot de passe
            login = glade_zeph.get_widget('login_zeph').get_text()
            passwd = glade_zeph.get_widget('pwd_zeph').get_text()
            addr =  glade_zeph.get_widget('adresse_zeph').get_text()
            self.addr_zephir = addr
            self.login_zephir = login
            zephir = ServerProxy('https://%s:%s@%s:7080' % (login, passwd, addr))
            try:
                data = zephir.modules.get_module()
                assert data[0] == 1
                for mod in data[1]:
                    # Choix des versions EOLE à importer (4 => Eole-2.2)
                    if mod['version'] == 4:
                        self.zephir_modules[mod['id']] = (mod['libelle'], False)
                    else:
                        self.zephir_modules[mod['id']] = (mod['libelle'], True)
                self.zephir = zephir
                self.glade.get_widget('zephir_send').set_sensitive(True)
                self.glade.get_widget('zephir_get').set_sensitive(True)
                self.glade.get_widget('zephir_select').set_sensitive(True)
                self.glade.get_widget('zephir_disconnect').set_sensitive(True)
                self.glade.get_widget('zephir_import').set_sensitive(True)
                # chargement de la liste des établissements disponibles
                code, etabs = self.zephir.etabs.get_etab()
                if code != 1:
                    self._error_window(unicode('Erreur de recherche des établissements sur zephir : %s' % etabs, config.charset))
                    self.etabs = []
                else:
                    self.etabs = [etab['rne'] for etab in etabs]
                    self.etabs.sort()
                if id_serveur != -1 and self.zephir_server is None:
                    self.zephir_server = id_serveur
                    self.zephir_initialize()
            except:
                self._error_window('Echec de la connexion à Zephir')
        dial_zeph.destroy()

    def zephir_disconnect(self, *args):
        self.glade.get_widget('zephir_send').set_sensitive(False)
        self.glade.get_widget('zephir_get').set_sensitive(False)
        self.glade.get_widget('zephir_select').set_sensitive(False)
        self.glade.get_widget('zephir_disconnect').set_sensitive(False)
        self.glade.get_widget('zephir_import').set_sensitive(False)
        self.zephir = None
        self.zephir_rne = None
        self.zephir_server = None
        self.init_default_dict()
        self.main()
        self._refresh_display()

    def set_title(self, libelle, size=42):
        # met à jour le libellé au dessus de la zone de saisie
        label = self.glade.get_widget('label_title')
        label.set_markup('<span foreground="#a0569a" font_desc="monospace bold %s">%s</span>' % (str(size), libelle))

    def build_led(self, green = False):
        """leds vertes et rouges pour la validation des onglets"""
        led = gtk.Image()
        if green:
            led.set_from_stock(gtk.STOCK_YES, gtk.ICON_SIZE_BUTTON)
        else:
            led.set_from_stock(gtk.STOCK_NO, gtk.ICON_SIZE_BUTTON)
        return led

    def _quitter(self):
        """
        quitte l'application principale
        """
        gtk.main_quit()

    def _callback_load_defaults(self, *args):
        """
        charge dans les edits les valeurs par defaut pour la famille editée
        """
        if self.edited_family != '':
            text = "Voulez-vous vraiment charger les valeurs par defaut pour ce groupe ?"
            reponse = self._question_window(text)
            if reponse == 1:
                self._restore_defaults(family=self.edited_family)
                if self.edited_family in self.valid_families:
                    self.valid_families.remove(self.edited_family)
                self._update_buttons()
                self._clear_table()
                self._build_table(self.edited_family)
                self._update_display()

    def _callback_button_quit(self, *args):
        """
        click sur bouton quitter
        """
        #on a pas sauve avant de quitter
        if self.save_flag == False:
            widgets = gtk.glade.XML(self.interface_file,'dialog_confirm')
            dial_conf = widgets.get_widget('dialog_confirm')
            dial_label = widgets.get_widget('label_confirm')
            text = "Voulez vous sauver le fichier avant de quitter ?"
            dial_label.set_text(text)
            dial_conf.set_title("Confirmation de sortie")
            reponse = dial_conf.run()
            dial_conf.destroy()
            if reponse == gtk.RESPONSE_CANCEL:
                return True
            elif reponse == gtk.RESPONSE_YES:
                # sauvegarde du fichier
                self._callback_button_save(None)
        self._quitter()

    def set_filters(self, filedialog):
        current_dir = os.path.dirname(self.filename)
        filedialog.set_current_folder(current_dir)
        # nb : le raccourci sur le home_dir est créé automatiquement
        filedialog.add_shortcut_folder(config.configeoldir)
        filter = gtk.FileFilter()
        filter.set_name("Tous les fichiers")
        filter.add_pattern("*")
        filedialog.add_filter(filter)
        filter = gtk.FileFilter()
        filter.set_name("Fichiers de valeurs Creole")
        filter.add_pattern("*.eol")
        filedialog.add_filter(filter)
        filedialog.set_filter(filter)

    def _callback_button_save(self, widget):
        """
        click sur bouton sauver
        """
        # avant tout, on sauvegarde
        if not self._save_values(None):
            return False
        ok = True
        boutons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT)
        save_dlg = gtk.FileChooserDialog(action = gtk.FILE_CHOOSER_ACTION_SAVE, buttons = boutons)
        save_dlg.set_current_name(unicode(self.filename, 'UTF-8'))
        # on demande validation si le fichier existe
        save_dlg.set_do_overwrite_confirmation(True)
        self.set_filters(save_dlg)

        if save_dlg.run() == gtk.RESPONSE_ACCEPT:
            ok = True
            text = "Enregistrement dans le fichier selectionne impossible"
            try:
                out_file = save_dlg.get_filename()
                #erreurs dans les groupes, on restaure les valeurs précédentes
                if self._check_groups() == False:
                    save_dlg.destroy()
                    return False
                ok = self.eole_dico.save_values(out_file)
            except TypeEoleError, e:
                ok = False
                text = e.args[0]
            else:
                self.valid_families = self.eole_dico.family_order
                # update des onglets (led verte)
                for tab in self.button_stock.items():
                    tab[1].set_image(self.build_led(True))
                    tab[1].get_image().show()
                self.save_flag = True
            if ok == False:
                self._error_window(text)
            else:
                self.filename = out_file
                self.window.set_title('Configuration: %s' % self.filename)
                # si mode zephir, on propose la sauvegarde à distance
                if self.zephir is not None and self.zephir_server is not None:
                    if self._question_window("Envoyer la modification sur zephir ?", "Sauvegarde"):
                        self.zephir_send()
        save_dlg.destroy()

    def _callback_button_load(self, widget):
        """
        click sur bouton ouvrir
        """
        ok = True
        boutons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT)
        open_dlg = gtk.FileChooserDialog(action = gtk.FILE_CHOOSER_ACTION_OPEN, buttons = boutons)
        open_dlg.select_filename(unicode(self.filename, 'UTF-8'))
        self.set_filters(open_dlg)
        text = "Chargement depuis le fichier selectionne impossible"
        if open_dlg.run() == gtk.RESPONSE_ACCEPT:
            try:
                if self.eole_dico.load_values(open_dlg.get_filename(), check=False) == False:
                    ok = False
                else:
                    self.filename = open_dlg.get_filename()
                    self.window.set_title('Configuration: %s' % self.filename)
                    self.valid_families = []
                    self.var_status = {}
                    self._refresh_display()
            except Exception, e:
                ok = False
                text = "Erreur au chargement des valeurs :\n" + e.__class__.__name__ + ' : ' + str(e)
        open_dlg.destroy()
        if ok == False:
            self._error_window(text)

    def _callback_button_import(self, widget):
        """
        click sur bouton import Creole1
        """
        boutons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT)
        open_dlg = gtk.FileChooserDialog(action = gtk.FILE_CHOOSER_ACTION_OPEN, buttons = boutons)
        open_dlg.select_filename(unicode(self.filename, 'UTF-8'))
        self.set_filters(open_dlg)
        #errors = []
        #warnings = []
        if open_dlg.run() == gtk.RESPONSE_ACCEPT:
            try:
                self.eole_dico.import_values(open_dlg.get_filename())
            except Exception, e:
                open_dlg.destroy()
                self._error_window(str(e))
                return False
            # mise à jour de l'affichage
            self.valid_families = []
            self._refresh_display()
            msg = "Mise à niveau terminée\nVérifiez les valeurs"
            MessageWindow("Erreur", unicode(str(msg), config.charset), self.window, "ok")
        open_dlg.destroy()

    def _callback_button_defaults(self, widget):
        """
        click sur bouton charger defauts
        """
        text = "Voulez-vous vraiment charger les valeurs par defaut?"
        reponse = self._question_window(text)
        if reponse == 1:
            #on met la valeur par defaut en interne
            self._restore_defaults()
            #on met a jour l'affichage
            self.valid_families = []
            self._refresh_display()

    def _callback_notebook_switcher(self, notebook, page, num, name):
        """
            gestion des onglets dans le Notebook
            (le conteneur des variables multiples)
        """
        maxi = notebook.get_n_pages()-1
        if num == maxi and num != 0:
            # nouvel onglet nécessaire
            curnum = notebook.get_current_page()
            table = notebook.get_nth_page(curnum)
            numtab = 0
            # recuperation de l'index du widget courant (pour copie des valeurs)
            for tab in self.creolevars[name]['tables']:
                if tab == table:
                    break
                numtab += 1
            self._add_notebook_page(notebook, name, numtab)
            notebook.show_all()
            notebook.set_current_page(maxi)
            # passe l'onglet a rouge
            self.modified_flag = True
            if self.edited_family in self.valid_families:
                self.valid_families.remove(self.edited_family)
            self.button_stock[self.edited_family].set_image(self.build_led())
            self.button_stock[self.edited_family].get_image().show()
            self._disable_edits()
            notebook.stop_emission("switch-page")

    def _add_notebook_page(self, notebook, name, numtab=None, index=None):
        """ajoute un onglet à un notebook existant
        numtab : numero d'une page existante à répliquer
        """
        len_widgs = len(self.creolevars[name]['widgets'])
        if numtab is not None:
            # copie de la valeur d'un onglet existant (ajout de nouvelle page)
            saisie_texte = self._get_text(self.creolevars[name]['widgets'][numtab].text_box)
        else:
            # récupération de la valeur de la variable creole
            saisie_texte = self.eole_dico.get_value(name)[len_widgs]
        cpt = 0
        self._add_text_field(None, name, saisie_texte, None, notebook, cpt=cpt, num=len_widgs)
        # gestion des variables esclaves si il y en a
        slaves = self.eole_dico.variables[name].get_slaves()
        # on parcourt croelevars pour respecter l'ordre (ref #3346)
        for slave in self.eole_dico.families[self.edited_family]['vars']:
            if slave in slaves:
                cpt += 1
                if numtab is not None:
                    saisie_texte = self._get_text(self.creolevars[slave]['widgets'][numtab].text_box)
                else:
                    saisie_texte = self.eole_dico.get_value(slave)[len_widgs]
                self._add_text_field(None, slave, saisie_texte, None, notebook, cpt=cpt, num=len_widgs)

    def refresh_notebook(self, var):
        """met à jour l'affichage d'un notebook en fonction des valeurs des variables
        (sans recalculer toute la page)
        """
        #récupération du notebook et des tables / widgets
        notebook = self.creolevars[var].get('notebook', None)
        if notebook:
            # on bloque temporairement le switch des pages
            notebook.handler_block(notebook.switch_callb)
            # vidage de tous les onglets (sauf l'onglet '+')
            for num in range(notebook.get_n_pages() - 1):
                try:
                    notebook.remove_page(0)
                    self._remove_text_field(var, 0)
                except Exception, e:
                    self._error_window(str(e.args[0]))
            # création des nouveaux onglets
            for num in range(len(self.eole_dico.get_value(var))):
                self._add_notebook_page(notebook, var, index=num)
            # maj affichage
            notebook.handler_unblock(notebook.switch_callb)
            notebook.show_all()
            notebook.set_current_page(0)

    def zephir_get(self, *args):
        # si mode zephir : on récupère les valeurs automatiquement
        ok = True
        if self.zephir is not None:
            try:
                if self.zephir_server is None:
                    # le n° du serveur a configurer n'est pas connu, on le demande :
                    self._error_window('Aucun serveur sélectionné')
                    return False
                code, res = self.zephir.serveurs.get_dico(self.zephir_server, 'modif_config')
                # chargement des valeurs
                values = eval(res[-1])
                self.eole_dico.store._sections = values
                self.eole_dico._load_values(check=False)
                text = "Erreur de chargement depuis zephir"
            except Exception, e:
                code = 0
                text = str(e)
            if code == 0:
                ok = False
            if ok == False:
                self._error_window(text)
            self.valid_families = []
            self._refresh_display()
        else:
            return False

    def zephir_send(self, widget=None, *args):
        if self.zephir is not None:
            # vérification des valeurs en cours d'édition
            if not self._save_values(None):
                return False
            if self.zephir_server is None:
                self._error_window('Aucun serveur sélectionné')
                return False
            else:
                ok = True
                try:
                    # si le fichier existe, on demande confirmation
                    ok = self.eole_dico.save_values('/dev/null')
                    # erreurs dans les groupes, on restaure les valeurs precedentes
                    if self._check_groups() == False:
                        return False
                except TypeEoleError, e:
                    ok = False
                    text = e.args[0]
                else:
                    self.valid_families = self.eole_dico.family_order
                    # update des onglets (led verte)
                    for tab in self.button_stock.items():
                        tab[1].set_image(self.build_led(True))
                        tab[1].get_image().show()
                    self.save_flag = True
                if ok == False:
                    self._error_window(text)
                else:
                    # configuration validée, envoi sur zephir
                    conf = unicode(str(self.eole_dico.store._sections), config.charset)
                    try:
                        code, msg = self.zephir.serveurs.save_conf(self.zephir_server, [conf])
                    except Exception, e:
                        code = 0
                        msg = str(e)
                    if code == 0:
                        text = msg
                        self._error_window(text)
                    else:
                        # si lancé depuis le bouton d'envoi, demande de sauvegarde locale ?
                        if widget is not None:
                            if self._question_window("Sauvegarder également en local ?", "Envoi Zephir"):
                                try:
                                    self.eole_dico.save_values(config.configeol)
                                except:
                                    self._error_window(unicode("Erreur de mise à jour de la configuration locale (%s)"% config.configeol, config.charset))
                        # sauvegarde réussie
                        MessageWindow(unicode("Envoi à Zephir", config.charset), unicode("configuration sauvegardée sur Zephir", config.charset), self.window).getrc()
        return False

    def zephir_select(self, *args):
        if self.module_import != None:
            if self.glade_select != None:
                # changement de mode (import/édition) : on reset le dialogue de sélection
                self.glade_select.get_widget('combo_rne').get_child().set_text('')
                tree = self.glade_select.get_widget('tree_etab')
                model = tree.get_model()
                model.clear()
            self.module_import = None
        self._zephir_select()

    def _zephir_select(self):
        if self.zephir is None:
            self.zephir_connect()
            if self.zephir is None:
                self._error_window(unicode("Vous n'êtes pas authentifié sur Zephir", config.charset))
                return False
        if self.dial_select is None:
            self.glade_select = gtk.glade.XML(self.interface_file,'dialog_select')
            self.dial_select = self.glade_select.get_widget('dialog_select')
            tree = self.glade_select.get_widget('tree_etab')
            tree.set_model(gtk.ListStore(gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_STRING))
            tree.append_column(gtk.TreeViewColumn('id', gtk.CellRendererText(), text=0))
            tree.append_column(gtk.TreeViewColumn('module', gtk.CellRendererText(), text=1))
            tree.append_column(gtk.TreeViewColumn('libelle', gtk.CellRendererText(), text=2))
            # récupération des données accessibles sur zephir
            self._fill_dial_select()
        else:
            tree = self.glade_select.get_widget('tree_etab')
        resp = self.dial_select.run()
        if resp == gtk.RESPONSE_OK:
            # récupération du serveur sélectionné
            model, iter_sel = tree.get_selection().get_selected()
            if iter_sel != None:
                self.zephir_server = int(model.get_value(iter_sel, 0))
                if self.module_import == None:
                    # édition d'un serveur NG quelconque de zephir
                    self.zephir_initialize()
                else:
                    # import d'une configuration creole1
                    self.zephir_import_conf()
        self.dial_select.hide()

    def _fill_dial_select(self):
        # remplissage de la combobox
        combo = self.glade_select.get_widget('combo_rne')
        combo.set_model(gtk.ListStore(gobject.TYPE_STRING))
        combo.set_text_column(0)
        for etab in self.etabs:
            combo.append_text(etab)
        combo.get_child().connect('changed', self.update_rne)

    def update_rne(self, *args):
        combo = self.glade_select.get_widget('combo_rne')
        rne = combo.get_active_text()
        if rne in self.etabs:
            self.zephir_rne = rne
            code, serveurs = self.zephir.serveurs.groupe_serveur({'rne':rne})
            if code != 1:
                self._error_window(unicode('Erreur de lecture des informations sur zephir : %s' % serveurs, config.charset))
            else:
                tree = self.glade_select.get_widget('tree_etab')
                model = tree.get_model()
                model.clear()
                for serveur in serveurs:
                    serveur_ok = False
                    if self.module_import == None:
                        # mode normal, on n'affiche que les serveurs eole2
                        if self.zephir_modules[int(serveur['module_actuel'])][1] == True:
                            serveur_ok = True
                    else:
                        # si on est en mode import, on ne prend que les modules eole1 du même type
                        if self.zephir_modules[int(serveur['module_actuel'])][0].startswith(self.module_import):
                            if self.zephir_modules[int(serveur['module_actuel'])][1] == False:
                                serveur_ok = True
                    if serveur_ok == True:
                        model.append([serveur['id'], self.zephir_modules[int(serveur['module_actuel'])][0], serveur['libelle']])

    def zephir_import(self, *args):
        """fonction d'import des données d'un serveur creole1
        """
        if self.module_import == None:
            self.init_default_dict()
            self.main()
            self._refresh_display()
            # changement de mode (import/édition) : on reset le dialogue de sélection
            if self.glade_select != None:
                self.glade_select.get_widget('combo_rne').get_child().set_text('')
                tree = self.glade_select.get_widget('tree_etab')
                model = tree.get_model()
                model.clear()
            self.module_import = config.eole_module
        self._zephir_select()

    def zephir_initialize(self):
        try:
            module_ok = self.zephir_title()
            if module_ok == True:
                code, res = self.zephir.serveurs.get_dico(self.zephir_server, 'config')
                if self.zephir_server != id_serveur:
                    self.eole_dico = EoleDict(templatedir=config.templatedir)
                    # si on édite un autre serveur, on prend les dictionnaires présents sur zephir
                    dicos = [base64.decodestring(dico_zeph) for dico_zeph in res[:-1]]
                    # chargement de tous les dictionnaires disponibles
                    for xml_dict in dicos:
                        self.eole_dico.read_string(xml_dict)
                self.modified_flag = False
                self.edited_family = ''
                #construit et affiche les boutons a gauche
                self._build_buttons()
                # rafraichit l'affichage
                self.valid_families = []
                for fam in self.eole_dico.family_order:
                    if not self.eole_dico.families[fam]['hidden'] == True:
                        if self.mode == 'normal' and fam not in self.eole_dico.normal_families:
                            continue
                        self.button_stock[fam].set_active(True)
                        break
                self.zephir_get()
                self._refresh_display()
        except Exception, e:
            self._error_window('Erreur de lecture des informations sur zephir \n(%s)' % str(e))
        return False

    def zephir_import_conf(self):
        """
        import d'une configuration Creole de version antérieure
        depuis Zéphir
        """
        # récupération du dictionnaire de valeurs sur Zéphir
        module_ok = self.zephir_title()
        if module_ok == True:
            code, res = self.zephir.serveurs.get_dico(self.zephir_server, 'modif_config')
            from ConfigParser import ConfigParser
            cfgp = ConfigParser()
            cfgp._sections = eval(res[-1])
            # fichier temporaire de conversion
            descr, temp_conffile = tempfile.mkstemp(suffix=".eol")
            f_tmp = open(temp_conffile, 'w')
            cfgp.write(f_tmp)
            f_tmp.close()
            try:
                self.eole_dico.import_values(temp_conffile)
            except Exception, e:
                self._error_window(str(e))
                return False
            os.unlink(temp_conffile)
            # mise à jour de l'affichage
            self.dial_select.hide()
            self.valid_families = []
            self._refresh_display()
            msg = "Mise à niveau terminée\nVérifiez les valeurs"
            MessageWindow("Erreur", unicode(str(msg), config.charset), self.window, "ok")

    def zephir_title(self, *args):
        # infos sur le serveur
        try:
            code, info_serv = self.zephir.serveurs.get_serveur(self.zephir_server)
        except ProtocolError:
            self._error_window('Accès au serveur %s non autorisé' % self.zephir_server)
        except Exception, e:
            self._error_window('Erreur de lecture des informations sur zephir \n(%s)' % str(e))
        else:
            libelle = info_serv[0]['libelle']
            id_mod = info_serv[0]['module_actuel']
            if id_mod in self.zephir_modules.keys():
                if self.module_import is None:
                    if self.zephir_modules[id_mod][1] == True:
                        server_title = """%s - %s (%s)\n %s""" % (info_serv[0]['rne'], info_serv[0]['libelle'], self.zephir_server, self.zephir_modules[id_mod][0])
                        self.set_title(server_title, size=20)
                        return True
                else:
                    if self.zephir_modules[id_mod][1] == False:
                        return True
                self._error_window('Module du serveur %s non géré' % self.zephir_server)
        return False

    def _refresh_display(self, * args):
        if self.edited_family == '':
            self.edited_family = self.eole_dico.family_order[0]
            self._untoggle_buttons(self.edited_family)

        self._clear_table()
        self._build_table(self.edited_family)
        #on met a jour l'affichage
        self._update_display()
        self.save_flag = True
        self._update_buttons()
        #active/desactive boutons de famille selon champ hidden
        self._disable_buttons()
        #active/desactive les labels/edits selon leur champ hidden
        self._disable_edits()
        self._update_display()

    def _callback_combobox_changed(self, widget, name):
        """
        callback quand on change la valeur d'une combobox
        """
        if type(widget) == gtk.ComboBoxEntry:
            # les comboboxentry sont traités de la mm façon que les entry
            return False
        val = widget.get_active_text()
        try:
            self.eole_dico.set_value(name, val)
            self.var_status[name] = ""
        except Exception, e:
            self._error_window(str(e.args[0]))
            return False
        #active/desactive boutons de famille selon champ hidden
        self._disable_buttons()
        #active/desactive les labels/edits selon leur champ hidden
        self._disable_edits()
        self.modified_flag = True
        #self._update_display()
        if self.edited_family in self.valid_families:
            self.valid_families.remove(self.edited_family)
        self.button_stock[self.edited_family].set_image(self.build_led())
        self.button_stock[self.edited_family].get_image().show()

    def _callback_modified_textbox(self, *args):
        """
        modification d'une case
        """
        if self.edited_family in self.valid_families:
            self.valid_families.remove(self.edited_family)
        self.button_stock[self.edited_family].set_image(self.build_led())
        self.button_stock[self.edited_family].get_image().show()
        self.modified_flag = True

    def _update_display(self):
        """
        Met a jour l'affichage des edits selon les valeurs courantes
        """
        errors = []
        for i, value in self.creolevars.items():
            cpt_saisie = 0
            for wid in value['widgets']:
                saisie = wid.text_box.get_children()[0]
                text = self._display_text(i)
                if text == []:
                    # sauvegarde du fichier
                    text = ['']
                if type(saisie) == gtk.Entry:
                    # widget Entry
                    saisie.set_text(text[cpt_saisie])
                    cpt_saisie += 1
                elif type(saisie) == gtk.ComboBoxEntry:
                    # widget ComboBoxEntry
                    saisie.get_child().set_text(text[cpt_saisie])
                    cpt_saisie += 1
                else:
                    # widget ComboBox
                    model = saisie.get_model()
                    cpt = 0
                    while cpt < len(model):
                        if model[cpt][0] == text[cpt_saisie]:
                            break
                        cpt += 1
                    if cpt == len(model):
                        errors.append(str(i))
                    saisie.set_active(cpt)
                    cpt_saisie += 1
        if errors != []:
            text = "Variable(s) inconnue : \n%s" % "\n".join(errors)
            self._error_window(text)


    def _restore_defaults(self, family=None, var=None):
        """
        charge les valeurs par defaut
        family : modifications sur une famille seulement
        """
        # création de la liste des variables à réinitialiser
        liste_vars = []
        if var:
            # variables d'un onglet
            liste_vars.append(var)
            if var in self.eole_dico.groups:
                liste_vars.extend(self.eole_dico.groups[var])
        elif family:
            # variables d'une famille
            liste_vars.extend(self.eole_dico.families[family]['vars'])
        else:
            # reset global
            for family in self.eole_dico.family_order:
                liste_vars.extend(self.eole_dico.families[family]['vars'])
        # première passe sur toutes les familles
        pass1 = []
        for key in liste_vars:
            def_val = self.eole_dico.variables[key].get_value(default=True)
            self.eole_dico.variables[key].set_value(def_val, force=True)
        # on crée une liste des valeurs de référence pour vérifier que tout est réinitialisé
        for key in liste_vars:
            if key in self.eole_dico.depends or self.eole_dico.variables[key].slave != False:
                # seulement pour les variables dépendantes d'autres
                # et les variables 'slave'
                # (pas pour les variables auto dans tous les cas)
                pass1.append(str(self.eole_dico.variables[key].val))
        # passes_suivantes, tant qu'une passe provoque des changements
        new_pass = True
        while new_pass:
            pass2 = []
            # nouvelle passe pour les valeurs calculées
            for key in liste_vars:
                def_val = self.eole_dico.variables[key].get_value(default=True)
                self.eole_dico.variables[key].set_value(def_val, force=True)
                try:
                    self.eole_dico.check_conditions(key, force=True)
                except Exception, e:
                    self._error_window(str(e))
            for key in liste_vars:
                if key in self.eole_dico.depends:
                    pass2.append(str(self.eole_dico.variables[key].val))
            if pass1 != pass2:
                # on met à jour la liste de valeurs pour la prochaine itération
                pass1 = []
                pass1.extend(pass2)
            else:
                new_pass = False

    def _update_buttons(self):
        """updates led on all buttons"""
        for cle, button in self.button_stock.items():
            if cle in self.valid_families:
                button.set_image(self.build_led(True))
            else:
                button.set_image(self.build_led())
            button.get_image().show()

    def _build_buttons(self):
        """
        construit les boutons des familles a gauche et les stocke dans un dictionnaire
        """
        # réninitialisation pour le changement de mode
        for child in self.vbox_button.get_children():
            self.vbox_button.remove(child)

        # on affiche les onglets, dans l'ordre de  eole_dico.family_order, les familles expert en dernier
        button_order =  []
        for fam in self.eole_dico.family_order:
            if fam in self.eole_dico.normal_families:
                button_order.append(fam)
        for fam in self.eole_dico.family_order:
            if fam not in self.eole_dico.normal_families:
                button_order.append(fam)
        for cle in button_order:
            if self.mode == 'normal' and cle not in self.eole_dico.normal_families:
                continue
            if cle in self.eole_dico.normal_families:
                name = cle.capitalize()
            else:
                name = cle.capitalize() + '*'
            # FIXME : remplacer l'étoile par un texte en couleur ;)
            button = gtk.ToggleButton(name, None)
            button.set_alignment(0, -1)
            button.show()
            self.button_stock[cle] = button
            self.vbox_button.pack_start(button)
            #connecte le callback pour quand on clique sur le bouton
            button.connect("toggled", self._callback_toggle_button, cle)
            #ajoute l'aide sur les boutons
            self._set_button_tooltip(cle, button)
        # voyants pour la validation des données
        self._update_buttons()
        self.update_button_win()
        self.vbox_button.show_all()

    def update_button_win(self):
        """redimensionne la fenêtre des boutons (familles)
        """
        scroll_win = self.glade.get_widget('scrolledwindow_family')
        if self.mode == 'expert':
            scroll_win.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
        else:
            scroll_win.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)

    def _callback_toggle_button(self, widget, name):
        """
        callback lors de l'appui sur un bouton de famille a gauche
        On va vider puis construire la table a droite selon les variables de la famille
        @param widget: widget qui declenche le callback
        @param name: nom de la famille sur laquelle on clique
        """
        if widget.get_active() == True:
            # éviter le 'first time'
            if self.edited_family != '' and self.modified_flag == True:
                reponse = self._question_window("Valider les modifications ?")
                if reponse == 1:
                    if not self._save_values(None):
                        self._untoggle_buttons(self.edited_family)
                        return False
                else:
                    # réinitialisation des modifications
                    # non validées dans les variables multi
                    for var in self.creolevars.keys():
                        if 'val_orig' in self.creolevars.get(var, {}):
                            self.eole_dico.variables[var].val = self.creolevars[var]['val_orig']['val']
                            self.eole_dico.variables[var].valeole = self.creolevars[var]['val_orig']['valeole']
                            self.eole_dico.variables[var].valdefault = self.creolevars[var]['val_orig']['valdefault']
                            self.eole_dico.variables[var].valprec = self.creolevars[var]['val_orig']['valprec']
                            self.eole_dico.variables[var].hidden = self.creolevars[var]['val_orig']['hidden']
                            del(self.creolevars[var]['val_orig'])

            self.modified_flag = False
            #on 'reset' la scrollbar de la fenêtre principale
            scrollwin = self.glade.get_widget('scrolledwindow1')
            scrollwin.do_scroll_child(scrollwin, gtk.SCROLL_START, False)
            #detruit les widgets contenus dans la table
            self._clear_table()
            #reconstruit la table selon la famille en cours
            self._build_table(name)
            #ne met actif que le bouton sur lequel on a appuye
            self._untoggle_buttons(name)
            #on stocke le nom de la famille éditée
            self.edited_family = name

    def _display_text(self, var, default=False, calculated=False):
        """
        renvoie la chaine de valeur associee a var (au lieu d'une liste, comme ca on peut afficher)
        """
        if calculated == False:
            val_actu = self.eole_dico.variables[var].get_value(default)
        else:
            val_actu = self.eole_dico.variables[var].get_calculated_value()

        return val_actu

    def _callback_focus_in_edit(self, widget, event, var, force=False):
        """
        focus rentre dans edit
        """
        if len(self.eole_dico.variables[var].fills) > 0:
            if widget.get_text() == '' or force == True:
                #sauvegarde
                sauv_var = {}
                self._push_vals(sauv_var)
                cpt = -1
                #pour trouver l'index de l'edit dans la liste des edits de chaque variable
                for i in self.creolevars[var]['widgets']:
                    cpt += 1
                    #1966
                    if i.text_box.get_children()[0] == widget:
                        break

                for key in self.creolevars.keys():
                    tmp = []
                    try:
                        for saisie in self.creolevars[key]['widgets']:
                            chaine = self._get_text(saisie.text_box)
                            tmp.append(chaine)
                        self.eole_dico.variables[key].val = tmp
                        self.eole_dico.variables[key].valdefault = ['']
                        self.eole_dico.variables[key].valeole = ['']
                    except Exception, e:
                        pass
                try:
                    text = self._display_text(var, calculated=True)
                except:
                    text = []
                if text != []:
                    if len(text) == 1:
                        val = text[0]
                    else:
                        val = text[cpt]
                else:
                    val = ''
                widget.set_text(val)
                #restauration
                self._pop_vals(sauv_var)

    def _set_label_tooltip(self, variable, label, edit):
        """
        Met un tooltip d'aide sur un label + edit
        """
        if self.eole_dico.helps['variables'].has_key(variable):
            text = self.eole_dico.helps['variables'][variable]
            #tip = gtk.Tooltip()
            #tip.set_text(label, text)
            #tip.set_text(edit, text)
            #tip.enable()
            label.set_tooltip_text(text)
            edit.set_tooltip_text(text)

    def _set_button_tooltip(self, family, widget):
        """
        Met un tooltip d'aide sur un widget(bouton)
        """
        if self.eole_dico.helps['families'].has_key(family):
            text = self.eole_dico.helps['families'][family]
            #tip = gtk.Tooltip()
            #tip.set_text(widget, text)
            #tip.enable()
            widget.set_tooltip_text(text)

    def _check_has_enum(self, var):
        checks = self.eole_dico.variables[var].checks
        for check in checks:
            if check[0] == 'valid_enum':
                checkval = "True"
                for param in check[1]:
                    if param['name'] == 'checkval':
                        checkval =  param['value']
                liste = check[1][0]['value']
                return liste, checkval
        return None, None

    def _remove_tab(self, button, name, tab_child):
        """Fonction de suppression de page"""
        lgt = len([widgt for widgt in self.creolevars[name]['widgets'] if widgt!=None])
        if lgt > 1:
            notebook = self.creolevars[name]['notebook']
            maxi = notebook.get_n_pages()-1
            # on récupère l'onglet correspondant à la table passée en paramètre
            num = notebook.page_num(tab_child)
            if num+1 == maxi and notebook.get_current_page() == num:
                # si on supprime le dernier onglet,
                # évite de se retrouver sur l'onglet '+'
                notebook.set_current_page(num-1)
            notebook.remove_page(num)
            try:
                self._remove_text_field(name, num)
            except Exception, e:
                self._error_window(str(e.args[0]))
            self._reorder_tabs(name)
            self.modified_flag = True

    def _remove_text_field(self, name, index):
        # destruction des widgets référencés
        # et mise à jour de la valeur des variables

        # XXX FIXME : widgets pas recréés ?
        self.creolevars[name]['widgets'].pop(index).destroy()
        table = self.creolevars[name]['tables'].pop(index)
        if table: table.destroy()
        # vérification des variables groupées
        if self.eole_dico.groups.has_key(name):
            for slave in self.eole_dico.groups[name]:
                self._remove_text_field(slave, index)

    def _reorder_tabs(self, name, label=None):
        # actualise les libellés du notebook
        notebook = self.creolevars[name]['notebook']
        index = 0
        for table in self.creolevars[name]['tables']:
            libelle = label or '' + ' Valeur ' + str(index + 1)
            tab_label, tab_button = notebook.get_tab_label(table).get_children()
            tab_label.set_text(libelle)
            index += 1

    def add_label_to_tab(self, label, var, index, table):
        """Fonction pour ajouter un bouton fermer"""
        tab_box = gtk.HBox(False, 2)
        if label == None:
            label = ''
        tab_label = gtk.Label(label+' Valeur '+str(index+1))
        tab_button = gtk.Button()
        tab_button.connect('clicked', self._remove_tab, var, table)

        icon_box = gtk.HBox(False, 0)
        image = gtk.Image()
        image.set_from_stock(gtk.STOCK_CLOSE, gtk.ICON_SIZE_MENU)
        gtk.Button.set_relief(tab_button, gtk.RELIEF_NONE)
        #On affecte à w et h les dimensions
        #On modifie ces dimensions
        settings = gtk.Widget.get_settings(tab_button)
        (w, h) = gtk.icon_size_lookup_for_settings(settings, gtk.ICON_SIZE_MENU)
        gtk.Widget.set_size_request(tab_button, w + 4, h + 4)
        image.show()

        icon_box.pack_start(image, True, False, 0)
        tab_button.add(icon_box)

        tab_box.pack_start(tab_label, False)
        tab_box.pack_start(tab_button, False)
        tab_box.show_all()
        return tab_box

    def _build_label(self, var):
        span = '<span'
        if self.eole_dico.variables[var].mode == 'expert':
            span += ' foreground="%s"' % EXPERT_COLOR
        if 'obligatoire' in [check[0] for check in self.eole_dico.variables[var].checks]:
            span += ' font_desc="bold"'
        texte = span + '> ' + self.eole_dico.variables[var].description
        if self.mode == 'expert':
            texte += '\n ( '+var+' )'
        texte += '</span>'
        if self.var_status.has_key(var):
            if self.var_status[var] != "":
                if self.mode != 'expert':
                    texte += "\n"
                # le message d'erreur démarre au 1er ":"
                texte += '<span foreground="#dd2222">%s</span>' % ':'.join(self.var_status[var].split(':')[1:])
        label = gtk.Label()
        label.set_markup(texte)
        label.set_justify(gtk.JUSTIFY_CENTER)
        # autorisation du copier/coller sans focus
        label.set_selectable(True)
        label.unset_flags("GTK_CAN_FOCUS")
        label.set_line_wrap(True)
        # largeur des libelles des variables
        #label.set_size_request(530, -1)
        label.show()
        return label

    def _build_line(self, var, sep_box=None, notebook=None, cpt=0, num=0):
        # boite des champs de texte
        text_box = gtk.VBox()
        view = self._build_label(var)
        label_box = gtk.VBox()
        button_box = gtk.HBox()
        label_box.pack_start(view, expand=True)
        label_box.show_all()
        widgets = Widgets(text_box=text_box, label_box=label_box, button_box=button_box, sep_box=sep_box)
        master = self.eole_dico.variables[var].slave
        tables = []
        if master != False and master in self.creolevars:
            #c'est un slave
            notebook = self.creolevars[master]['notebook']
        if self.creolevars.has_key(var):
            self.creolevars[var]['widgets'].append(widgets)
        else:
            self.creolevars[var] = {'widgets': [widgets], 'notebook': notebook, 'tables': tables}
        text_box.show_all()
#        button_hbox = gtk.HBox()

        # boutons "valeur précédente" et "valeur par défaut"
        button_prec = gtk.Button('Prec')
        button_def  = gtk.Button('Def')
        button_prec.set_size_request(40, 30)
        button_def.set_size_request(40, 30)
        button_prec.show()
        button_def.show()
        button_prec.unset_flags("GTK_CAN_FOCUS")
        button_def.unset_flags("GTK_CAN_FOCUS")
        button_box.pack_start(button_prec, False, False)
        button_box.pack_start(button_def, False, False)

        #button_box.pack_start(button_hbox, False, False)
        button_box.show_all()
        #ajoute l'aide (tooltip)
        self._set_label_tooltip(var, view, text_box)
        pos = num
        if self.creolevars[var]['notebook'] == None:
            #si pas de group
            table = self.table
            notebook = None
        else:
            #si group
            if master == False:
                # variable maître, on crée une nouvelle page
                pos = notebook.get_n_pages() - 1
                notebook = self.creolevars[var]['notebook']
                table = gtk.Table()
                notebook.show_all()
                notebook.insert_page(table, self.add_label_to_tab(None, var, num, table), position=pos)
            else:
                # variable esclave
                table = self.creolevars[master]['tables'][num]
                tables = [table]
            self.creolevars[var]['tables'].append(table)

        button_prec.connect('clicked', self._set_prec_value, var, pos)
        button_def.connect('clicked', self._set_def_value, var, pos)

        table.attach(label_box, 0, 1, cpt, cpt+1, gtk.FILL|gtk.EXPAND, 0, 0, 0)
        # attache les champ de saisie
        table.attach(text_box, 1, 2, cpt, cpt+1, gtk.FILL|gtk.EXPAND, 0, 0, 2)
        table.attach(button_box, 2, 3, cpt, cpt+1, 0, 0, 3, 0)

    def _build_table(self, family):
        """
        Construit la table selon les variables de la famille en cours
        @param family: famille en cours
        """
        cpt = 0
        #on ajuste la taille de la table selon la famille
        self.table.resize(len(self.eole_dico.families[family]['vars']), 3)
        self.creolevars = {}
        for var in self.eole_dico.families[family]['vars']:
            # recherche des séparateurs
            sep_box = None
            if self.eole_dico.separators.has_key(var):
                if self.eole_dico.variables[var].mode != 'expert' or self.mode == 'expert':
                    separator = self.eole_dico.separators[var]
                    # Création d'une boite avec le séparateur
                    sep_box = gtk.HBox()
                    sep_box.pack_start(gtk.HSeparator(), True, True)
                    texte_sep = '<span foreground="#a0569a" font_desc="monospace bold 12">%s</span>' % separator[0]
                    label_sep = gtk.Label()
                    label_sep.set_markup(texte_sep)
                    sep_box.pack_start(label_sep, False, False)
                    sep_box.pack_start(gtk.HSeparator(), True, True)
                    sep_box.show_all()
                    self.table.attach(sep_box, 0, 3, cpt, cpt+1, gtk.FILL|gtk.EXPAND, 0, 0, 4)
                    cpt += 1
            # texte a afficher dans le label
            # pour les variables obligatoires, on affiche le texte en gras
            notebook = None
            if self.eole_dico.variables[var].multi == True:
                if self.eole_dico.variables[var].mode == 'normal' or self.mode == 'expert':
                    notebook = gtk.Notebook()
                    self.table.attach(notebook, 0, 3, cpt, cpt+1, gtk.FILL|gtk.EXPAND, 0, 4, 4)
                    croix = gtk.Label('+')
                    croix2 = gtk.Label('+')
                    notebook.append_page(croix2, croix)
            values = self.eole_dico.variables[var].get_value()
            if values == []:
                values = ['']
            num = 0
            for val in values:
                self._add_text_field(None, var, val, sep_box, notebook, cpt, num=num)
                num += 1
            if self.eole_dico.variables[var].multi == True or self.eole_dico.variables[var].slave != False:
                # on stocke les valeurs pour un retour en arrière si pas de validation
                if var in self.creolevars:
                    self.creolevars[var]['val_orig'] = {'val':self.eole_dico.variables[var].val,
                                                        'valprec':self.eole_dico.variables[var].valprec,
                                                        'valdefault':self.eole_dico.variables[var].valdefault,
                                                        'valeole':self.eole_dico.variables[var].valeole,
                                                        'hidden':self.eole_dico.variables[var].hidden,
                                                       }
            if notebook != None:
                notebook.show_all()
                # pas de scroll sur les onglets
                notebook.connect('scroll_event', lambda *args: True)
                notebook.switch_callb = notebook.connect('switch-page', self._callback_notebook_switcher, var)
            cpt += 1

        self._disable_edits()
        #self.table.set_homogeneous(True)
        self.table.show()

    def _add_text_field(self, widget, var, value = None, sep_box=None, notebook=None, cpt=0, num=0):
        if self.eole_dico.variables[var].mode == 'expert' and self.mode != 'expert':
            return
        enum = self._check_has_enum(var)
        # index dans la liste des valeurs
        self._build_line(var, sep_box=sep_box, notebook=notebook, cpt=cpt, num=num)
        index = len(self.creolevars[var]['widgets'])
        #si ce n'est pas une enumération
        if enum[0] == None:
            try:
                if value != None:
                    saisie_texte = value
                else:
                    saisie_texte = self._display_text(var)[0]
            except IndexError:
                saisie_texte = ''
            #champ de saisie
            saisie = gtk.Entry()
            saisie.set_size_request(200, 30)
            #texte a afficher dans le champ de saisie
            saisie.set_text(saisie_texte)
            saisie.connect('focus-in-event', self._callback_focus_in_edit, var)
        else:
            if enum[1] == "False":
                # on permet la saisie de valeur non présentes dans la liste
                saisie = gtk.combo_box_entry_new_text()
            else:
                saisie = gtk.combo_box_new_text()
            cpt_t = 0
            index_txt = 0
            if value != None:
                saisie_texte = value
            else:
                if index == -1:
                    saisie_texte = self._display_text(var)[0]
                else:
                    if len(text_box.get_children()) == 1:
                        input = text_box.get_children()[0]
                    else:
                        input = text_box.get_children()[-1]
                    model = input.get_model()
                    if enum[1] == "False":
                        saisie_texte = input.get_child().get_text()
                    else:
                        saisie_texte = input.get_active_text()
            if enum[1] == "False":
                for text in eval(enum[0]):
                    saisie.append_text(text)
                saisie.get_child().set_text(saisie_texte)
            else:
                for text in eval(enum[0]):
                    saisie.append_text(text)
                    if text == saisie_texte:
                        index_txt = cpt_t
                    cpt_t += 1
                saisie.set_active(index_txt)
            #if type(saisie) in (gtk.ComboBoxEntry, gtk.Combo):
            saisie.set_size_request(200, 30)
            #else:
            #    saisie.set_size_request(305, 35)
        text_box = self.creolevars[var]['widgets'][-1].text_box
        if len(text_box.get_children()) != 0:
            raise Exception('text_box non vide')
        text_box.pack_start(saisie, False, False)
        # champ modifié => action
        if type(saisie) != gtk.ComboBox:
            #gtk.ComboBoxEntry et gtk.Entry
            saisie.connect('changed', self._callback_modified_textbox)
        else:
            saisie.connect('changed', self._callback_combobox_changed, var)
        saisie.show()
        text_box.show()

    def _set_prec_value(self, widget, var, index):
        """
        On remplit le champ texte avec la valeur précédente
        """
        notebook = self.creolevars[var].get('notebook', None)
        if notebook and (self.eole_dico.variables[var].slave or var in self.eole_dico.groups):
            # variable multiple: on ne change que la valeur courante dans le cas d'un groupe
            pos = notebook.get_current_page()
        else:
            pos = 0
        val_prec = self.eole_dico.get_prec_value(var)
        if pos != None:
            if len (val_prec) < index+1:
                val = ''
            else:
                val = val_prec[index]
        else:
            val = val_prec[0]
            pos = 0
        saisie = self.creolevars[var]['widgets'][pos].text_box.get_children()[0]
        if type(saisie) in (gtk.ComboBoxEntry, gtk.ComboBox):
            model = saisie.get_model()
            cur_iter = model.get_iter_root()
            while cur_iter:
                if model.get_value(cur_iter, 0) == val:
                    saisie.set_active_iter(cur_iter)
                    break
                cur_iter = model.iter_next(cur_iter)
        else:
            saisie.set_text(val)

    def _set_def_value(self, widget, var, index):
        """
        On remplit le champ texte avec la valeur par défaut
        """
        notebook = self.creolevars[var].get('notebook', None)
        set_multi = 0
        if self.eole_dico.variables[var].multi:
            # proposer un retour au nombre de valeurs par défaut (sauf variable esclave)
            if var in self.eole_dico.groups:
                question_multi = "Réinitialiser toute la section ?"
            else:
                question_multi = "Réinitialiser toutes les valeurs ?"
            set_multi = self._question_window("%s\n (Si non: juste cette valeur)" % question_multi)
        if set_multi == 1:
            # variable maître, on réinitialise tout le groupe ?
            self._restore_defaults(var=var)
            self.refresh_notebook(var)
        else:
            if notebook:
                # variable multiple: on ne change que la valeur courante
                pos = notebook.get_current_page()
            else:
                pos = 0
            val_def = self.eole_dico.get_value(var, default=True)
            if pos != None:
                if len (val_def) < index+1:
                    val = ''
                else:
                    val = val_def[index]
            else:
                val = val_def[0]
                pos = 0
            saisie = self.creolevars[var]['widgets'][pos].text_box.get_children()[0]
            if len(self.eole_dico.variables[var].fills) > 0:
                self._callback_focus_in_edit(saisie, None, var, True)
            else:
                if type(saisie) in (gtk.ComboBoxEntry, gtk.ComboBox):
                    model = saisie.get_model()
                    cur_iter = model.get_iter_root()
                    while cur_iter:
                        if model.get_value(cur_iter, 0) == val:
                            saisie.set_active_iter(cur_iter)
                            break
                        cur_iter = model.iter_next(cur_iter)
                else:
                    saisie.set_text(val)
        self.modified_flag = True
        if self.edited_family in self.valid_families:
            self.valid_families.remove(self.edited_family)
        self._update_buttons()
        #active/desactive boutons de famille selon champ hidden
        self._disable_buttons()
        #active/desactive les labels/edits selon leur champ hidden
        self._disable_edits()

    def _clear_table(self):
        """
        Detruit tous les widgets de la table
        """
        self.table.foreach(self._widget_destroy)

    def _widget_destroy(self, widget):
        """
        Detruit le widget passe en parametre
        """
        widget.destroy()

    def _untoggle_buttons(self, name):
        """
        ne met que le bouton passe en parametre en mode "toggled"
        """
        for i in self.button_stock.keys():
            if i != name:
                self.button_stock[i].set_active(False)
            else:
                self.button_stock[i].set_active(True)


    def _disable_buttons(self):
        """
        Active et desactive les boutons selon leur champ hidden
        """
        for family, value in self.eole_dico.families.items():
            if self.mode == 'normal' and family not in self.eole_dico.normal_families:
                continue
            #si pas de variable dans la famille
            hidden = value['hidden']
            if not hidden:
                if self.eole_dico.families[family]['vars']  == []:
                    hidden = True
                #si expert la variable est forcement dedans
                else:
                    if self.mode == 'normal':
                        has_var = False
                        for var in self.eole_dico.families[family]['vars']:
                            if self.eole_dico.variables[var].mode != 'expert':
                                has_var = True
                                break
                        if not has_var:
                            hidden = True
            if hidden == True:
                #self.button_stock[family].set_sensitive(False)
                self.button_stock[family].hide()
            else:
                #self.button_stock[family].set_sensitive(True)
                self.button_stock[family].show()

    def _disable_edits(self):
        """
        Active et desactive les labels et edits selon leur champ hidden
        """
        for key, value in self.creolevars.items():
            if self.eole_dico.variables[key].hidden == True:
                for widget in value['widgets']:
                    if widget != None:
                        if self.hide_vars == False:
                            widget.text_box.set_sensitive(False)
                            widget.label_box.set_sensitive(False)
                            widget.button_box.set_sensitive(False)
                            if widget.sep_box != None:
                                widget.sep_box.set_sensitive(False)
                        else:
                            widget.text_box.hide()
                            widget.label_box.hide()
                            widget.button_box.hide()
                            if widget.sep_box != None:
                                if self.eole_dico.separators[key][1]:
                                    # never_hidden='True'
                                    widget.sep_box.show()
                                else:
                                    widget.sep_box.hide()
            else:
                for widget in value['widgets']:
                    if widget != None:
                        if self.hide_vars == False:
                            widget.text_box.set_sensitive(True)
                            widget.label_box.set_sensitive(True)
                            widget.button_box.set_sensitive(True)
                            if widget.sep_box != None:
                                widget.sep_box.set_sensitive(True)
                        else:
                            widget.text_box.show()
                            widget.label_box.show()
                            widget.button_box.show()
                            if widget.sep_box != None:
                                widget.sep_box.show()

            if self.creolevars[key]['notebook'] != None:
                for table in self.creolevars[key]['tables']:
                    if table != None:
                        break
                if table != None:
                    display = False
                    for wid in table.get_children():
                    #    if type(wid.get_children()[0]) != gtk.Label:
                        if self.hide_vars == False:
                            if wid.get_sensitive() == True:
                                display = True
                                break
                        else:
                            if wid.get_property('visible') == True:
                                display = True
                                break
                    if display:
                        if self.hide_vars == False:
                            self.creolevars[key]['notebook'].set_sensitive(True)
                        else:
                            self.creolevars[key]['notebook'].show()
                    else:
                        if self.hide_vars == False:
                            self.creolevars[key]['notebook'].set_sensitive(False)
                        else:
                            self.creolevars[key]['notebook'].hide()
                for table in self.creolevars[key]['tables']:
                    if table != None:
                        break
    def _get_text(self, text_box):
        """
            récupère le texte dans le widget text_box
        """
        text_box = text_box.get_children()[0]
        if type(text_box) == gtk.Entry:
            saisie = text_box.get_text()
        else:
            model = text_box.get_model()
            if type(text_box) == gtk.ComboBoxEntry:
                saisie = text_box.get_child().get_text()
            else:
                saisie = text_box.get_active_text()
        return saisie.strip()

    def _save_values(self, widget):
        """
        sauve les valeurs contenues dans les champ de saisie de la table
        """
        if self.edited_family != '':
            #sauvegarde les valeurs actuelles pour eventuellement les restaurer
            val_sauv = {}
            self._push_vals(val_sauv)
            erreurs = []
            for var in self.creolevars.keys():
                val = []
                for wid in self.creolevars[var]['widgets']:
                    if wid != None:
                        saisie = self._get_text(wid.text_box)
                        val.append(saisie)
                        try:
                            self.eole_dico.set_value(var, val)
                            self.var_status[var] = ''
                        except Exception, e:
                            erreurs.append(e.args[0])
                            # on tag la variable pour l'afficher en rouge
                            self.var_status[var] = e.args[0]
                if self.creolevars[var].has_key('val_orig'):
                    del(self.creolevars[var]['val_orig'])

            if erreurs != []:
                self.button_stock[self.edited_family].set_image(self.build_led())
                self.button_stock[self.edited_family].get_image().show()
                if self.edited_family in self.valid_families:
                    self.valid_families.remove(self.edited_family)
                self._error_window("\n".join(erreurs))
                # mise à jour des libellés d'erreur
                for var, var_stat in self.var_status.items():
                    # pour les variables actuellement à l'écran seulement
                    if var_stat != "" and var in self.creolevars:
                        for wid in self.creolevars[var]['widgets']:
                            if wid != None:
                                label_box = wid.label_box
                                label_box.get_children()[0].destroy()
                                view = self._build_label(var)
                                label_box.pack_start(view, False, False)
                                label_box.show()
                # pas de refresh pour éviter de perdre les valeurs ajoutées
                # self._refresh_display()
                return False
            else:
                self.valid_families.append(self.edited_family)
                self.button_stock[self.edited_family].set_image(self.build_led(True))
                self.button_stock[self.edited_family].get_image().show()
                self._refresh_display()

            #active/desactive boutons de famille selon champ hidden
            self._disable_buttons()
            #active/desactive les labels/edits selon leur champ hidden
            self._disable_edits()
            #desactive le flag de sauvegarde
            self.save_flag = False
            self.modified_flag = False

        return True

    def _push_vals(self, dico):
        """
        sauvegarde les valeurs actuelles des variables de la famille actuelle
        """
        for var in self.creolevars.keys():
            dico[var] = {'val': self.eole_dico.variables[var].val}
            dico[var]['valprec'] = self.eole_dico.variables[var].valprec
            dico[var]['valdefault'] = self.eole_dico.variables[var].valdefault
            dico[var]['hidden'] = self.eole_dico.variables[var].hidden
            dico[var]['valeole'] = self.eole_dico.variables[var].valeole

    def _pop_vals(self, dico):
        """
        restaure les valeurs des variables passees en parametre
        """
        for var in dico.keys():
            self.eole_dico.variables[var].val = dico[var]['val']
            self.eole_dico.variables[var].valprec = dico[var]['valprec']
            self.eole_dico.variables[var].valdefault = dico[var]['valdefault']
            self.eole_dico.variables[var].hidden = dico[var]['hidden']
            self.eole_dico.variables[var].valeole = dico[var]['valeole']


    def _check_groups(self):
        """
        verifie que les conditions de groupes sont respectees
        i.e que tous les slaves d'un master sont remplis
        """
        #key: nom du master
        hidden_vars = []
        for name, fam in self.eole_dico.families.items():
            if fam['hidden'] == True:
                hidden_vars.extend(fam['vars'])
        # avant de vérifier, on recalcule les éventuels 'fills' des variables laissées vides
        for varname in self.eole_dico.variables.keys():
            var = self.eole_dico.variables[varname]
            if var.val == [] or var.val == ['']:
                var.val = var.get_value(default=True)
        #value: liste des slaves
        for key, value in self.eole_dico.groups.items():
            if key not in hidden_vars:
                #si master a une valeur
                if self.eole_dico.variables[key].val != []:
                    lg_key = len(self.eole_dico.variables[key].val)
                    for slave in value:
                        # slave hidden : non traité ici
                        if self.eole_dico.variables[slave].hidden:
                            continue
                        lg_slave = len(self.eole_dico.variables[slave].val)
                        if self.eole_dico.variables[slave].val == [] or lg_key != lg_slave:
                            text = 'Probleme de dependance entre '+key+' et '+slave+'. '+slave+' doit avoir une valeur.'
                            text += '\nOu '+slave+' doit avoir le meme nombre de valeurs que '+key
                            text += '\nEnregistrement non effectue'
                            self._error_window(text)
                            return False
        return True

    def _error_window(self, text, title='Erreur'):
        """
        affiche un message d'erreur
        """
        if type(text) != unicode:
            text = unicode(text, config.charset)
        text = text.encode("utf-8")
        MessageWindow (title, text, self.window, type="error")

    def _question_window(self, text, title='Question'):
        """
        Pose une question a l'utilisateur
        """
        return MessageWindow(title, text, self.window, type='yesno').getrc()

    def main(self):
        """
        lance l'interface
        """
        #construit et affiche les boutons a gauche
        self._build_buttons()
        #desactive les boutons dont le champ hidden est a True
        self._disable_buttons()
        self._disable_edits()
        #affiche la première page disponible
        for fam in self.eole_dico.family_order:
            if not self.eole_dico.families[fam]['hidden'] == True:
                if self.mode == 'normal' and fam not in self.eole_dico.normal_families:
                    continue
                self.button_stock[fam].set_active(True)
                break

        self.window.show()
        # self.window.maximize()

    def _change_mode(self, widget):
        """
        changement de mode
        """
        # fausse alerte !
        if widget.active :
            if self.mode == 'normal':
                return
        else:
            if self.mode == 'expert':
                return

        if self.modified_flag == True:
            # validation des changements ?
            reponse = self._question_window("Valider les modifications ?")
            if reponse == 1:
                if not self._save_values(None):
                    # erreur : on se remet dans l'ancien mode
                    if widget.active :
                        widget.set_active(False)
                    else:
                        widget.set_active(True)
                    return True

        # comme neuf
        self.modified_flag = False
        self.edited_family = ""

        # changement officiel
        if widget.active:
            self.mode = 'normal'
        else:
            self.mode = 'expert'
        self.main()


    def _change_affvar(self, widget):
        """
        changement de mode d'affichage des variables cachées
        """
        # fausse alerte !
        if widget.active :
            if self.hide_vars == False:
                return
        else:
            if self.hide_vars == True:
                return

        if self.modified_flag == True:
            # validation des changements ?
            reponse = self._question_window("Valider les modifications ?")
            if reponse == 1:
                if not self._save_values(None):
                    # erreur : on se remet dans l'ancien mode
                    if widget.active :
                        widget.set_active(False)
                    else:
                        widget.set_active(True)
                    return True

        self.modified_flag = False

        if widget.active:
            self.hide_vars = False
        else:
            self.hide_vars = True

        # réaffiche la page actuelle
        for fam in self.eole_dico.family_order:
            if fam == self.edited_family:
                self._callback_toggle_button(self.button_stock[fam], fam)
                break
        self.window.show()
        # self.window.maximize()


class MessageWindow:
    def getrc (self):
        return self.rc

    def __init__ (self, title, text, window, type="ok", default=None, custom_buttons=None, custom_icon=None):

        self.rc = None

        if type == 'ok':
            buttons = gtk.BUTTONS_OK
            style = gtk.MESSAGE_INFO
        elif type == 'warning':
            buttons = gtk.BUTTONS_OK
            style = gtk.MESSAGE_WARNING
        elif type == 'okcancel':
            buttons = gtk.BUTTONS_OK_CANCEL
            style = gtk.MESSAGE_WARNING
        elif type == 'yesno':
            buttons = gtk.BUTTONS_YES_NO
            style = gtk.MESSAGE_QUESTION
        elif type == 'error':
            buttons = gtk.BUTTONS_OK
            style = gtk.MESSAGE_ERROR
        elif type == 'custom':
            buttons = gtk.BUTTONS_NONE
            style = gtk.MESSAGE_QUESTION

        if custom_icon == "warning":
            style = gtk.MESSAGE_WARNING
        elif custom_icon == "question":
            style = gtk.MESSAGE_QUESTION
        elif custom_icon == "error":
            style = gtk.MESSAGE_ERROR
        elif custom_icon == "info":
            style = gtk.MESSAGE_INFO

        dialog = gtk.MessageDialog(window, 0, style, buttons, text)

        dialog.set_position (gtk.WIN_POS_CENTER)
        dialog.show_all ()

        rc = dialog.run()

        if rc == gtk.RESPONSE_OK or rc == gtk.RESPONSE_YES:
            self.rc = 1
        elif (rc == gtk.RESPONSE_CANCEL or rc == gtk.RESPONSE_NO
            or rc == gtk.RESPONSE_CLOSE):
            self.rc = 0
        elif rc == gtk.RESPONSE_DELETE_EVENT:
            self.rc = 0
        else:
            self.rc = rc
        dialog.destroy()

def parse_cmdline():
#    parser = optparse.OptionParser(usage=__doc__)
    parser = optparse.OptionParser("Usage: gen_config.py [-w] <repertoires> <fichier.eol>", version = "%prog")

    parser.add_option('-w', '--window-mode', dest = "window_mode", action = "store_true",
                  help=u"Désactive l'exécution en plein-écran")

    parser.add_option("-d", "--dictionnary-dirs", dest="dirs", default=",".join(config.eoledirs),
                  help=u"liste de répertoires contenant le ou les dictionnaires XML (séparateur \",\" sans espaces)")

    parser.add_option('-t', '--title', dest = "title", default=None,
                  help=u"Spécifie un titre")

    (options, args) = parser.parse_args()
    if args.count('txt'):
        args.remove('txt')
    if len(args) > 1:
        parser.error("Ne peut ouvrir qu'un fichier à la fois")
    return options, args

def get_dirs_list(dirs):
    return [i.strip() for i in dirs.split(',')]

def exiterr(msg):
    MessageWindow("Erreur", unicode(str(msg), config.charset), None, "error")

    sys.exit(1)

if __name__ == '__main__':
    # récupération de répertoires additionnels passés en argument
    options, args = parse_cmdline()

    if args:
        arg = args[0]
        if not arg.endswith('.eol'):
            exiterr('Le fichier à ouvrir doit avoir l\'extension ".eol"')
        if not os.path.isfile(arg):
            exiterr('Fichier "%s" non trouvé'%arg)
        config_file = os.path.abspath(arg)
    else:
        config_file = None

    dirs = get_dirs_list(options.dirs)
    try:
        # initialisation de l'interface
        gui = ConfigGui(GLADE_FILE, dirs, window_mode=options.window_mode,
                        filename=config_file, title=options.title)
        # options pour forcer l'affichage des icônes dans les boutons
        # settings = gtk.settings_get_default()
        # settings.props.gtk_button_images = True
        gui.main()
        gtk.main()
    except Exception, e:
        MessageWindow("Erreur", unicode(str(e), config.charset), None, "error")
        sys.exit(1)

