#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
    creole lint main module
"""
import types
import sys
import re
from glob import glob
from os.path import join, basename, isfile, expanduser, abspath

from creole.cfgparser import EoleDict
from creole import config
from creole import eosfunc
from pyeole.process import system_code
from creole.lint.normalize import is_correct

# variables internes aux dictionnaires
DICO_TEST_VARS = ['test_', 'tmp_']

starttoken = '%'
varstarttoken = '%%'
builts=[u'ArithmeticError', u'AssertionError', u'AttributeError',
    u'BaseException', u'DeprecationWarning', u'EOFError', u'Ellipsis',
    u'EnvironmentError', u'Exception', u'False', u'FloatingPointError',
    u'FutureWarning', u'GeneratorExit', u'IOError', u'ImportError',
    u'ImportWarning', u'IndentationError', u'IndexError', u'KeyError',
    u'KeyboardInterrupt', u'LookupError', u'MemoryError', u'NameError',
    u'None', u'NotImplemented', u'NotImplementedError', u'OSError',
    u'OverflowError', u'PendingDeprecationWarning', u'ReferenceError',
    u'RuntimeError', u'RuntimeWarning', u'StandardError',
    u'StopIteration', u'SyntaxError', u'SyntaxWarning', u'SystemError',
    u'SystemExit', u'TabError', u'True', u'TypeError',
    u'UnboundLocalError', u'UnicodeDecodeError', u'UnicodeEncodeError',
    u'UnicodeError', u'UnicodeTranslateError', u'UnicodeWarning',
    u'UserWarning', u'ValueError', u'Warning', u'ZeroDivisionError',
    u'_', u'__debug__', u'__doc__', u'__import__', u'__name__', u'abs',
    u'all', u'any', u'apply', u'basestring', u'bool', u'buffer',
    u'callable', u'chr', u'classmethod', u'cmp', u'coerce', u'compile',
    u'complex', u'copyright', u'credits', u'delattr', u'dict', u'dir',
    u'divmod', u'enumerate', u'eval', u'execfile', u'exit', u'file',
    u'filter', u'float', u'frozenset', u'getattr', u'globals',
    u'hasattr', u'hash', u'help', u'hex', u'id', u'input', u'int',
    u'intern', u'isinstance', u'issubclass', u'iter', u'len',
    u'license', u'list', u'locals', u'long', u'map', u'max', u'min',
    u'object', u'oct', u'open', u'ord', u'pow', u'property', u'quit',
    u'range', u'raw_input', u'reduce', u'reload', u'repr', u'reversed',
    u'round', u'set', u'setattr', u'slice', u'sorted', u'staticmethod',
    u'str', u'sum', u'super', u'tuple', u'type', u'unichr', u'unicode',
    u'vars', u'xrange', u'zip', u'adresse_ip_br0', u'adresse_netmask_br0',
    u'adresse_broadcast_br0', u'adresse_network_br0']
builts.append(u'is_defined')
builts.append(u'index')
builts.append(u'keys')
builts.append(u'values')
builts.append(u'split')
builts.append(u'value')
builts.append(u'lower')
for func in dir(eosfunc):
    if not func.startswith('_'):
        builts.append(unicode(func, 'utf-8'))

class TmplVar():
    def __init__(self, name, fd, line):
        self.name = name
        self.location = []
        self.add_location(fd, line)

    def add_location(self, fd, line):
        fd = basename(fd)
        self.location.append((fd, line+1))

    def set_location(self, location):
        self.location = location

    def get_location(self):
        return self.location

class CreoleLinter:
    """Base class for linters, collects creole vars and templates
    **has to be launched once and only once**
    """
    class __impl:
        warnno = 1
        warncomment = "Undefined comment"
        """ Implementation of the singleton interface """
        def set_config(self, eoledirs=None, distrib_dir=None, conffile=None, dtddir=None):
            if eoledirs != None and type(eoledirs) != list:
                raise TypeError('eoledirs doit être une liste')
            if distrib_dir != None and type(distrib_dir) != str:
                raise TypeError('distrib_dir doit être une string')
            if self.distrib_dir != None or self.eoledirs != None:
                sys.stderr('Tentative de redefinition de distrib_dir ou eoledirs')
            if distrib_dir == None:
                self.distrib_dir = config.distrib_dir
            else:
                self.distrib_dir = distrib_dir
            if eoledirs == None:
                self.eoledirs = config.eoledirs
            else:
                self.eoledirs = eoledirs
            if dtddir == None:
                self.dtddir = config.dtddir
            else:
                self.dtddir = dtddir
            self.exclude_var = []
            self.pkgname = None
            #lire dans un fichier
            if conffile != None:
                if isfile(conffile):
                    dico = {}
                    execfile(abspath(conffile), {}, dico)
                    if dico.has_key('dependencies'):
                        for dep in dico['dependencies']:
                            depfile = join(expanduser('~/.creolelint'), dep+'.conf')
                            if isfile(depfile):
                                dicodep = {}
                                execfile(depfile, dicodep)
                                if dicodep.has_key('vardico'):
                                    self.exclude_var.extend(dicodep['vardico'])
                    if dico.has_key('skip'):
                        for skip in dico['skip']:
                            func = skip[0]
                            var = skip[1]
                            filename = skip[2]
                            line = skip[3]
                            if not self.skip_var.has_key(func):
                                self.skip_var[func] = {}
                            if not self.skip_var[func].has_key(var):
                                self.skip_var[func][var] = []
                            self.skip_var[func][var].append((filename, line))
                    if dico.has_key('name'):
                        self.pkgname = dico['name']
                    if dico.has_key('level'):
                        self.conflevel = dico['level']

        def load_dics(self):
            if self.creoledic == None:
                self.creoledic = self._collect_vars_in_dicos()

        def load_tmpls(self):
            if self.tmplvars == None:
                self._collect_set_vars()
                self._collect_def_vars()
                self._collect_for_vars()
                self._collect_define_vars()
                self._collect_vars_in_tmplfiles()

        def get_dicos_name(self):
            if self.creoledic == None:
                raise Exception('Dictionnaire non chargé')
            dic = self.creoledic.variables.keys()
            dic.sort()
            return dic

        def get_dicos_files(self):
            if self.creoledic == None:
                raise Exception('Dictionnaire non chargé')
            dic = []
            for name in self.creoledic.files.keys():
                dic.append(basename(self.creoledic.files[name]['var'].filename))
            dic.sort()
            return dic

        def get_tmplvars_name(self):
            if self.tmplvars == None:
                raise Exception('Template non chargé')
            tmpl = self.tmplvars.keys()
            tmpl.sort()
            return tmpl

        def get_defvars_name(self):
            if self.defvars == None:
                raise Exception('Template non chargé')
            defv = self.defvars.keys()
            defv.sort()
            return defv

        def get_setvars_name(self):
            if self.setvars == None:
                raise Exception('Fonction non chargé')
            var = self.setvars.keys()
            var.sort()
            return var

        def get_forvars_name(self):
            if self.forvars == None:
                raise Exception('Fonction non chargé')
            var = self.forvars.keys()
            var.sort()
            return var

        def get_tmplvar(self, name):
            return self.tmplvars[name]

        def get_separators(self):
            return self.creoledic.separators

        def get_dico_file_names(self):
            if self.eoledirs == None or self.distrib_dir == None:
                raise Exception('Utiliser la methode set_config avant')
            ret = []
            for eoledir in self.eoledirs:
                eoledir=abspath(eoledir)
                ret.extend(glob(join(eoledir, '*.xml')))
            return ret

        def get_dtd(self):
            if self.dtddir == None:
                raise Exception('Utiliser la methode set_config avant')
            return join(self.dtddir, 'creole.dtd')

        def _collect_vars_in_dicos(self):
            if self.eoledirs == None or self.distrib_dir == None:
                raise Exception('Utiliser la methode set_config avant')
            dic = EoleDict(templatedir=self.distrib_dir, logfile='/dev/null', linter=True)
            dic.read_dir(self.eoledirs)
            return dic

        def _parse_tabs_in_dicos(self):
            if self.eoledirs == None or self.distrib_dir == None:
                raise Exception('Utiliser la methode set_config avant')
            tabs_in_dics = []
            for directory in self.eoledirs:
                for fname in glob(join(directory, "*.xml")):
                    fh = file(fname, 'r')
                    content = fh.read()
                    if '\t' in content:
                        tabs_in_dics.append(fname)
                    fh.close()
            return tabs_in_dics

        def _list_tmpl_files(self):
            return [fi for fi in glob(join(self.distrib_dir, '*'))
                           if not fi.startswith('.')
                           if not fi.endswith('~')]

        def _collect_var_in(self, dvar, var, fd, linenb, exists=False):
            if exists == True:
                test_builts=True
                if self.forvars.has_key(var):
                    #si deja en memoire
                    if (basename(fd), linenb+1) in self.forvars[var].location:
                        return
                    dvar = self.forvars
                    test_builts=False
                if self.setvars.has_key(var):
                    dvar = self.setvars
                    test_builts=False
                if self.defvars.has_key(var):
                    dvar = self.defvars
                    test_builts=False
                #test les builtsin seulement si variable
                if test_builts == True and unicode(var, 'utf-8') in builts:
                    #self.builtsvar.append(var)
                    return
            if dvar.has_key(var):
                dvar[var].add_location(fd=fd, line=linenb)
            else:
                dvar[var] = TmplVar(name=var, fd=fd, line=linenb)

        def _collect_vars_in(self, expr, dvar, fd, linenb, exists=False):
            varcreole = re.compile(varstarttoken+'([a-zA-Z0-9_\.{}]+)')
            varcreole2 = re.compile(varstarttoken+'(\w+)')
            varcreolebr = re.compile(varstarttoken+'{(\w+)}')
            varmulti = re.compile('(\w+)\.(\w+)')
            for var in varcreole.findall(expr):

                ret = varmulti.match(var)
                if ret != None:
                    #%%var.sousvar
                    self._collect_var_in(dvar, ret.group(1), fd, linenb, exists)
                    self._collect_var_in(dvar, ret.group(2), fd, linenb, exists)
                else:
                    #%%var
                    for var2 in varcreole2.findall(varstarttoken+var):
                        self._collect_var_in(dvar, var2, fd, linenb, exists)
                    #%%{var}
                    for var2 in varcreolebr.findall(varstarttoken+var):
                        self._collect_var_in(dvar, var2, fd, linenb, exists)

        def _collect_for_vars(self):
            if self.forvars == None:
                self.forvars = {}
                self.brokenfor = []
                pattern = re.compile(starttoken+'for (.+) in (.*)')
                for tmplfd in self._list_tmpl_files():
                    fh = open(tmplfd, 'r')
                    lines = fh.readlines()
                    length = len(lines)
                    for linenb in range(length):
                        line = lines[linenb]
                        ret = pattern.match(line.strip())
                        if ret != None:
                            self._collect_vars_in(ret.group(1), self.forvars, tmplfd, linenb)
                            if ret.group(1).strip()[:2] != varstarttoken:
                                self.brokenfor.append(TmplVar(name=line.strip(), fd=tmplfd, line=linenb))

                    fh.close()

        def _collect_def_vars(self):
            if self.defvars == None:
                self.defvars = {}
                pattern = re.compile(starttoken+'def (.*)\((.*)\)')
                for tmplfd in self._list_tmpl_files():
                    fh = open(tmplfd, 'r')
                    lines = fh.readlines()
                    length = len(lines)
                    for linenb in range(length):
                        line = lines[linenb]
                        ret = pattern.match(line)
                        if ret != None:
                            self._collect_var_in(self.defvars, ret.group(1).strip(), tmplfd, linenb)
                            self._collect_vars_in(ret.group(2), self.defvars, tmplfd, linenb)

                    fh.close()

        def _collect_set_vars(self):
            if self.setvars == None:
                self.setvars = {}
                pattern = re.compile(starttoken+'set (.*)=.*')
                for tmplfd in self._list_tmpl_files():
                    if not isfile(tmplfd):
                        raise Exception('fichier template déclaré mais inexistant %s'%tmplfd)
                    fh = open(tmplfd, 'r')
                    lines = fh.readlines()
                    length = len(lines)
                    for linenb in range(length):
                        line = lines[linenb]
                        ret = pattern.match(line)
                        if ret != None:
                            self._collect_vars_in(ret.group(1), self.setvars, tmplfd, linenb)

                    fh.close()

        def _collect_define_vars(self):
            if self.var_with_is_defined == None:
                self.var_with_is_defined = {}
                pattern = re.compile('%s(elif|if) %sis_defined\(\'(\w+)\'\) and'%(starttoken, varstarttoken))
                for tmplfd in self._list_tmpl_files():
                    fh = open(tmplfd, 'r')
                    lines = fh.readlines()
                    length = len(lines)
                    for linenb in range(length):
                        line = lines[linenb]
                        ret = pattern.match(line)
                        if ret != None:
                            self._collect_var_in(self.var_with_is_defined, ret.group(1), tmplfd, linenb)
                    fh.close()

        def _collect_vars_in_tmplfiles(self):
            if self.eoledirs == None or self.distrib_dir == None:
                raise Exception('Utiliser la methode set_config avant')
            # XXX ".eolvars" is a good placeholder for var names to be kept in touch
            self.tmplvars = {}
            for tmplfd in self._list_tmpl_files():
                fh = open(tmplfd, 'r')
                lines = fh.readlines()
                length = len(lines)
                for linenb in range(length):
                    line = lines[linenb]
                    #FIXME: err
                    self._collect_vars_in(line, self.tmplvars, tmplfd, linenb, True)

                fh.close()
                # XXX cheetahVarStartToken may redefine the beginswith variable

    # storage for the instance reference
    __instance = None

    def __init__(self):
        """ Create singleton instance """
        # Check whether we already have an instance

        if CreoleLinter.__instance is None:
            # Create and remember instance
            CreoleLinter.__instance = CreoleLinter.__impl()
            self.distrib_dir = None
            self.eoledirs = None
            self.creoledic = None
            self.tmplvars = None
            self.forvars = None
            self.defvars = None
            self.setvars = None
            self.brokenfor = None
            self.var_with_is_defined = None
            self.exclude_var = None
            self.skip_var = {}
            self.conflevel = 'eole'

        # Store instance reference as the only member in the handle
        self.__dict__['_CreoleLinter__instance'] = CreoleLinter.__instance

    def __getattr__(self, attr):
        """ Delegate access to implementation """
        return getattr(self.__instance, attr)

    def __setattr__(self, attr, value):
        """ Delegate access to implementation """
        return setattr(self.__instance, attr, value)

class SaveItem(CreoleLinter):
    """
        eolvars not present in the dicos
    """
    name = 'save'
    warnno = 1
    warncomment = "Ne pas utiliser la fonction SaveItem comme un test"

    def process(self):
        self.load_dics()
        if self.pkgname == None:
            raise Exception('fichier creolelint.conf incomplet (name)')
        filename = join(expanduser('~/.creolelint'), self.pkgname+'.conf')
        fh = open(filename, 'w')
        fh.write('vardico = '+str(self.get_dicos_name())+'\n')
        fh.close()
        print('fichier sauvegardé')

    def check(self):
        return []

class ValidDTDItem(CreoleLinter):
    name='valid_dtd'
    warnno = 1
    warncomment = "DTD Non valide"

    def check(self):
        dtd = self.get_dtd()
        for filename in self.get_dico_file_names():
            system_code(['xmllint', '--noout', '--dtdvalid', dtd, filename])
        return []

class OrphansInDicosItem(CreoleLinter):
    """
        eolvars not present in the dicos
    """
    name = 'orphans_in_dicos'
    warnno = 8
    warncomment = "Variable dictionnaire non utilisée dans un template"

    def check(self):
        self.load_dics()
        self.load_tmpls()

        vars_name = set(self.get_dicos_name())
        tmplvars_name = set(self.get_tmplvars_name())
        only_in_dicos = vars_name - tmplvars_name
        ret = []
        skip_var = self.skip_var.get(self.name, {})
        for var in only_in_dicos:
            if skip_var.has_key(var):
                continue
            test_start = False
            for start in DICO_TEST_VARS:
                if var.startswith(start):
                    test_start = True
                    break
            if not test_start:
                ret.append(self.creoledic.variables[var])
        return ret

class OrphansInTmplItem(CreoleLinter):
    """
        eolvars not present in the templates
    """
    name = 'orphans_in_tmpl'
    warnno = 8
    warncomment = "Variable template non présente dans le dictionnaire"

    def check(self):
        self.load_dics()
        self.load_tmpls()

        vars_name = self.get_dicos_name()
        if self.exclude_var != None:
            vars_name.extend(self.exclude_var)
        vars_name=set(vars_name)
        tmplvars_name = set(self.get_tmplvars_name())
        only_in_tmpl = tmplvars_name - vars_name
        ret = []
        for var in only_in_tmpl:
            skipped_location = []
            for location in self.tmplvars[var].location:
                if self.skip_var.has_key(self.name) and self.skip_var[self.name].has_key(var):
                    if not location in self.skip_var[self.name][var]:
                        skipped_location.append(location)
                else:
                    skipped_location.append(location)
            if skipped_location != []:
                tmplvar = TmplVar(var, '', 0)
                tmplvar.set_location(skipped_location)
                ret.append(tmplvar)
            #results.append(self.tmplvars[var])
        return ret

class OrphansDefItem(CreoleLinter):
    name = 'orphans_def'
    warnno = 7
    warncomment = "Fonction définie mais non utilisée"

    def check(self):
        self.load_tmpls()
        results = []
        for defvar in self.get_defvars_name():
            defname = {}
            for filedesc, linenb in self.defvars[defvar].location:
                if not defname.has_key((defvar, filedesc)):
                    defname[(defvar, filedesc)]=linenb
                else:
                    defname[(defvar, filedesc)]="exists"
            for defvar, filedesc in defname.keys():
                if defname[(defvar, filedesc)] != "exists":
                    results.append(TmplVar(name=defvar, fd=filedesc, line=defname[(defvar, filedesc)]))

        return results

class OrphansSetItem(CreoleLinter):
    name = 'orphans_set'
    warnno = 7
    warncomment = "Variable définie dans le template mais non utilisée"

    def check(self):
        self.load_tmpls()
        results = []
        for setvar in self.get_setvars_name():
            setname = {}
            for filedesc, linenb in self.setvars[setvar].location:
                if setname.has_key((setvar, filedesc)) == False:
                    setname[(setvar, filedesc)]=linenb
                else:
                    setname[(setvar, filedesc)]="exists"

            for setvar, filedesc in setname.keys():
                if setname[(setvar, filedesc)] != "exists":
                    results.append(TmplVar(name=setvar, fd=filedesc, line=setname[(setvar, filedesc)]))

        return results

class OrphansForItem(CreoleLinter):
    name = 'orphans_for'
    warnno = 7
    warncomment = "Variable définie dans une boucle mais non utilisée"

    def check(self):
        self.load_tmpls()
        results = []
        for forvar in self.get_forvars_name():
            forname = {}
            for filedesc, linenb in self.forvars[forvar].location:
                if forname.has_key((forvar, filedesc)) == False:
                    forname[(forvar, filedesc)]=linenb
                else:
                    forname[(forvar, filedesc)]="exists"

            for forvar, filedesc in forname.keys():
                if forname[(forvar, filedesc)] != "exists":
                    results.append(TmplVar(name=forvar, fd=filedesc, line=forname[(forvar, filedesc)]))
        return results

class IsDefinedDefinedItem(CreoleLinter):
    """
    """
    name = 'is_defined_defined'
    warnno = 9
    warncomment = "Variable template avec condition is_defined alors qu'elle définie"

    def check(self):
        self.load_dics()
        self.load_tmpls()

        vars_name = set(self.get_dicos_name())
        is_defined_name = set(self.var_with_is_defined.keys())
        results = []
        for var in is_defined_name & vars_name:
            results.append(self.var_with_is_defined[var])
        return results

class OrphansDicosFilesItem(CreoleLinter):
    """
    """
    name = 'orphans_dicos_files'
    warnno = 8
    warncomment = "Template déclaré dans le dicos inexistant"

    def check(self):
        self.load_dics()

        dicos_files = []
        for filen in self.get_dicos_files():
            dicos_files.append(basename(filen))
        dicos_files = set(dicos_files)
        tmpl_files = []
        for filen in self._list_tmpl_files():
            tmpl_files.append(unicode(basename(filen), 'utf-8'))
        tmpl_files=set(tmpl_files)
        orphans = dicos_files - tmpl_files
        ret = []
        for var in orphans:
            if self.skip_var.has_key(self.name) and not self.skip_var[self.name].has_key(var):
                ret.append(var)
            else:
                ret.append(var)

        return ret

class OrphansTmplFilesItem(CreoleLinter):
    """
    """
    name = 'orphans_tmpl_files'
    warnno = 7
    warncomment = "Template non déclaré dans le dicos"

    def check(self):
        self.load_dics()

        dicos_files = []
        for filen in self.get_dicos_files():
            dicos_files.append(basename(filen))
        dicos_files = set(dicos_files)
        tmpl_files = []
        for filen in self._list_tmpl_files():
            tmpl_files.append(unicode(basename(filen), 'utf-8'))
        tmpl_files=set(tmpl_files)
        return tmpl_files - dicos_files

class WrongDicosNameItem(CreoleLinter):
    name = 'wrong_dicos_name'
    warnno = 7
    warncomment = "Dictionnaire avec un nom invalide"

    def check(self):
        if self.conflevel == 'common':
            pattern='(0'
        elif self.conflevel == 'conf':
            pattern='(1'
        elif self.conflevel == 'eole':
            pattern='(2'
        else:
            pattern='([3-9]'
        pattern+='[0-9]_[a-z0-9_]+)'
        cpattern = re.compile(pattern)
        ret = []
        for filename in self.get_dico_file_names():
            name = basename(filename)
            if cpattern.match(name) == None:
                ret.append(name)
        return ret


class DefineItem(CreoleLinter):
    """
        check for syntaxes
    """
    name = 'define'
    warnno = 4
    warncomment = "Redéfinition d'un variable d'un dictionnaire"

    def check(self):
        """
        verifie si une variable définie est une variable du dictionnaire
        """
        self.load_dics()
        self.load_tmpls()
        dicos = set(self.get_dicos_name())
        defv = set(self.get_defvars_name())
        ret=[]
        for var in defv & dicos:
            ret.append(self.defvars[var])
        return ret

class BuiltinsItem(CreoleLinter):
    """
        verifier si une variable de dico n'est pas dans l'espace de nommage
    """
    name = 'builtins'
    warnno = 4
    warncomment = "Variable identitique à une fonction python"

    def check(self):
        self.load_dics()
        self.load_tmpls()
        ret=[]
        #dans le dictionnaire
        for var in set(builts) & set(self.get_dicos_name()):
            ret.append(self.creoledic.variables[var])
        #dans les variables de template
        for var in set(builts) & set(self.get_tmplvars_name()):
            ret.append(self.tmplvars[var])
        #dans les boucles for
        for var in set(builts) & set(self.get_forvars_name()):
            ret.append(self.forvars[var])
        #dans la definition de variable dans un template
        for var in set(builts) & set(self.get_setvars_name()):
            ret.append(self.setvars[var])
        #dans les noms de fonction
        for var in set(builts) & set(self.get_defvars_name()):
            ret.append(self.defvars[var])
        return ret

class SyntaxForItem(CreoleLinter):
    """
        verifie la syntaxe de la ligne for
    """
    name = 'syntax_for'
    warnno = 1
    warncomment = "Syntaxe de la ligne for incorrect"
    def check(self):
        self.load_tmpls()
        return self.brokenfor

class SyntaxVarItem(CreoleLinter):
    """
        verifie les variables suivant la syntaxe de pattern
    """
    name = 'syntax_var'
    pattern='([a-z0-9][a-z0-9]+_[a-z0-9_]+)'
    warnno = 6
    warncomment = "La variable ne respecte pas la regexp %s" % pattern

    def check(self):
        cpattern = re.compile(self.pattern)
        self.load_dics()
        self.load_tmpls()
        ret=[]
        #dans le dictionnaire
        for var in self.get_dicos_name():
            if cpattern.match(var) == None:
                ret.append(self.creoledic.variables[var])
        #dans les variables de template
        for var in self.get_tmplvars_name():
            if cpattern.match(var) == None:
                skipped_location = []
                for location in self.tmplvars[var].location:
                    if self.skip_var.has_key(self.name) and self.skip_var[self.name].has_key(var):
                        if not location in self.skip_var[self.name][var]:
                            skipped_location.append(location)
                    else:
                        skipped_location.append(location)
                if skipped_location != []:
                    tmplvar = TmplVar(var, '', 0)
                    tmplvar.set_location(skipped_location)
                    ret.append(tmplvar)
                    #ret.append(self.tmplvars[var])
        #dans les boucles for
        for var in self.get_forvars_name():
            if cpattern.match(var) == None:
                ret.append(self.forvars[var])
        #dans la definition de variable dans un template
        for var in self.get_setvars_name():
            if cpattern.match(var) == None:
                ret.append(self.setvars[var])
        return ret

class ForbiddenTemplateVarItem(CreoleLinter):
    """
        vérifie la présence des noms de variable interdits dans les templates
    """
    name = 'syntax_var2'
    warnno = 6
    warncomment = "Nom de variable interdit dans un template"

    def check(self):
        #self.load_dics()
        self.load_tmpls()
        ret=[]
        #dans les variables de template
        for var in self.get_tmplvars_name():
            for start in DICO_TEST_VARS:
                if var.startswith(start):
                    ret.append(var)
                    break
        return ret

class SyntaxFunctionItem(CreoleLinter):
    """
        verifie les fonctions suivant la syntaxe de pattern
    """
    name = 'syntax_function'
    pattern='([a-z0-9][a-z0-9]+_[a-z0-9_]+)'
    warnno = 6
    warncomment = "La fonction ne respecte pas la regexp %s" % pattern

    def check(self):
        cpattern = re.compile(self.pattern)
        self.load_tmpls()
        ret=[]
        #dans les noms de fonction
        for var in self.get_defvars_name():
            if cpattern.match(var) == None:
                ret.append(self.defvars[var])
        return ret

class OrphansVarHelpItem(CreoleLinter):
    name = 'orphans_var_help'
    warnno = 3
    warncomment = "Aide définie dans le dictionnaire pour une variable inexistante"

    def check(self):
        self.load_dics()
        vars_name = set(self.get_dicos_name())
        vars_help = set(self.creoledic.helps['variables'].keys())
        #print vars_help
        only_in_help = vars_help - vars_name
        results = []
        for tmpl in only_in_help:
            results.append(tmpl)
        return results

class OrphansFamHelpItem(CreoleLinter):
    name = 'orphans_fam_help'
    warnno = 3
    warncomment = "Aide définie dans le dictionnaire pour une famille inexistante"

    def check(self):
        self.load_dics()
        vars_name = set(self.creoledic.families.keys())
        vars_help = set(self.creoledic.helps['families'].keys())
        only_in_help = vars_help - vars_name
        results = []
        for tmpl in only_in_help:
            results.append(tmpl)
        return results

class ValidVarLabelItem(CreoleLinter):
    name = 'valid_var_label'
    warnno = 6
    warncomment = "Libellé de variable non valide dans un dictionnaire"

    def check(self):
        ret = []
        for var in self.creoledic.variables.values():
            ret.extend(is_correct(var.description, var.name))
        return ret

class ValidSeparatorLabelItem(CreoleLinter):
    name = 'valid_separator_label'
    warnno = 6
    warncomment = "Libellé de séparateur non valide dans un dictionnaire"

    def check(self):
        ret = []
        for var, value in self.get_separators().items():
            ret.extend(is_correct(value[0], var))
        return ret

class ValidHelpLabelItem(CreoleLinter):
    name = 'valid_help_label'
    warnno = 6
    warncomment = "Libellé d'aide non valide dans un dictionnaire"

    def check(self):
        ret = []
        for var in self.creoledic.helps['variables'].keys():
            ret.extend(is_correct(self.creoledic.helps['variables'][var], var))
        for var in self.creoledic.helps['families'].keys():
            ret.extend(is_correct(self.creoledic.helps['families'][var], var))
        return ret

class ValidCheckEnumOuiNon(CreoleLinter):
    name = 'valid_check_enum_ouinon'
    warnno = 6
    warncomment = "Variable avec un valid_enum à oui/non au lieu du type oui/non"

    def check(self):
        ret = []
        for var, content in self.creoledic.variables.items():
            if str(type(content)) != "<class 'creole.typeole.OuiNon'>":
                for check in content.checks:
                    if check[0] == u'valid_enum':
                        for valid_enum in check[1]:
                            if valid_enum['value'].startswith('['):
                                eval_valid_enum = eval(valid_enum['value'])
                                if set(eval_valid_enum) == set(['non', 'oui']):
                                    ret.append(var)
        return ret

class ValidCheckEnumOnOff(CreoleLinter):
    name = 'valid_check_enum_onoff'
    warnno = 6
    warncomment = "Variable avec un valid_enum à on/off au lieu du type on/off"

    def check(self):
        ret = []
        for var, content in self.creoledic.variables.items():
            if str(type(content)) != "<class 'creole.typeole.OnOff'>":
                for check in content.checks:
                    if check[0] == u'valid_enum':
                        for valid_enum in check[1]:
                            if valid_enum['value'].startswith('['):
                                eval_valid_enum = eval(valid_enum['value'])
                                if set(eval_valid_enum) == set(['on', 'off']):
                                    ret.append(var)
        return ret

class ValidTabsInDicosItem(CreoleLinter):
    name = 'valid_tabs_in_dicos'
    warnno = 6
    warncomment = "Tabulation dans le dictionnaire au lieu de 4 espaces"

    def check(self):
        return self._parse_tabs_in_dicos()

def initialize(keyword):
    glob = globals()
    classitem = None
    for cls in glob:
        if cls.startswith('_') and type(glob[cls])!=types.ClassType and cls == 'CreoleLinter' and cls == 'TmplVar':
            continue
        if hasattr(glob[cls], 'name'):
            if glob[cls].name == keyword:
                classitem = glob[cls]
                break
    if classitem == None:
        raise Exception('test %s inconnu'%keyword)
    return classitem()

